import { useState, useEffect, useRef } from "react";
import { Table, Button } from "react-bootstrap";
import { useTranslation } from "react-i18next";
import { faEdit, faTrashAlt } from "@fortawesome/free-regular-svg-icons";
import { faArrowDown, faArrowUp, faPlus } from "@fortawesome/free-solid-svg-icons";
import { FontAwesomeIcon } from "@fortawesome/react-fontawesome";
import StaffModal from "../../components/StaffModal";
import { getBoatsFromApi, deleteBoat, addBoat, editBoat } from "../../api/boat";
import { getBoatTypesFromApi } from "../../api/boatTypes";
import { Boat, BoatType } from "../../types";
import { Form } from "react-bootstrap";
import StatusIcon from "../../components/StatusIcon";
import { Controller, useForm } from "react-hook-form";

function BoatManager() {
  let mounted = useRef(true);
  const { t } = useTranslation();
  const [editElement, setEditElement] = useState<Boat | undefined>(undefined);
  const [deleteElement, setDeleteElement] = useState<number>(-1);
  const [isAdding, setAdding] = useState<boolean>(false);
  const [boats, setBoats] = useState<Boat[]>([]);
  const [boatTypes, setBoatTypes] = useState<BoatType[]>([]);
  const [loading, setLoading] = useState<boolean>(false);
  const [boatNameSearch, setBoatNameSearch] = useState<string>("");
  const [boatTypeSearch, setBoatTypeSearch] = useState<string>("");
  const [boatStatusSearch, setBoatStatusSearch] = useState<number>(-1);
  //Default will be sorting by name descending
  const [sortByName, setSortByName] = useState<boolean>(true);
  const [sortByNameAsc, setSortByNameAsc] = useState<boolean>(false);
  const [sortByType, setSortByType] = useState<boolean>(false);
  const [sortByTypeAsc, setSortByTypeAsc] = useState<boolean>(false);
  const [sortByStatus, setSortByStatus] = useState<boolean>(false);
  const [sortByStatusAsc, setSortByStatusAsc] = useState<boolean>(false);

  const {
    control,
    handleSubmit,
    setValue,
    formState: { errors, isValid },
  } = useForm<Boat>({
    mode: "onBlur",
    reValidateMode: 'onChange'
  });

  const getBoats = async () => {
    const boats = await getBoatsFromApi();
    if (mounted) {
      setBoats(boats.result);
    }
  };
  async function getBoatTypes() {
    if (mounted) {
      setBoatTypes(await getBoatTypesFromApi());
    }
  }
  
  useEffect(() => {
    getBoats();
    getBoatTypes();
    return () => {
      mounted.current = false;
    }
  }, []);
  return (
    <div className="m-1 h-100">
      <Table responsive striped bordered hover>
        <thead>
          <tr>
            <th
            onClick={() => {
              setSortByName(true);
              setSortByNameAsc(!sortByNameAsc);
              setSortByType(false);
              setSortByStatus(false);
            }}>
              {t("boatManager.BoatName")} {sortByName && <FontAwesomeIcon icon={!sortByNameAsc ? faArrowDown : faArrowUp} />}
            </th>
            <th 
            onClick={() => {
              setSortByName(false);
              setSortByStatus(false);
              setSortByTypeAsc(!sortByTypeAsc);
              setSortByType(true);
            }}>
              {t("boatManager.BoatType")} {sortByType && <FontAwesomeIcon icon={!sortByTypeAsc ? faArrowDown : faArrowUp} />}
            </th>
            <th
            onClick={() => {
              setSortByName(false);
              setSortByType(false);
              setSortByStatusAsc(!sortByStatusAsc);
              setSortByStatus(true);
            }}>
              {t("boatManager.Status")} {sortByStatus && <FontAwesomeIcon icon={!sortByStatusAsc ? faArrowDown : faArrowUp} />}
            </th>
            <th></th>
          </tr>
          <tr>
            <th>
            <Form.Control 
                type="text" 
                className="form-control-sm" 
                placeholder="Search" 
                aria-label="Search" 
                onChange={(event) => {
                  setBoatNameSearch(event.target.value);
                }}
              />
            </th>
            <th>
            <Form.Control 
                type="text" 
                className="form-control-sm" 
                placeholder="Search" 
                aria-label="Search" 
                onChange={(event) => {
                  setBoatTypeSearch(event.target.value);
                }}
              />
            </th>
            <th>
            <Form.Select 
                className="form-control-sm" 
                placeholder="Search" 
                aria-label="Search" 
                onChange={(event) => {
                  setBoatStatusSearch(parseInt(event.target.value));
                }}
              >
                <option value={-1}> </option>
                <option value={0}>{t("boatManager.available")}</option>
                <option value={1}>{t("boatManager.locked")}</option>
              </Form.Select>
            </th>
          </tr>
        </thead>
        <tbody style={{ overflowY: "scroll", maxHeight: "100px" }}>
          {boats
          .sort((a,b) => {
            if(sortByName && sortByNameAsc) {
              console.log("Sort By Name ASC");
              return b.name.localeCompare(a.name);
            }
            //I know these are gross
            else if(sortByType && sortByTypeAsc) {
              let boatTypeA = (boatTypes.find(t => t.id === a.boattype));
              let boatTypeAStr = boatTypeA === undefined ? "" : boatTypeA.name;
              let boatTypeB = boatTypes.find(t => t.id === b.boattype);
              let boatTypeBStr = boatTypeB === undefined ? "" : boatTypeB.name;
              return boatTypeBStr.localeCompare(boatTypeAStr);
            }
            else if(sortByType && !sortByTypeAsc) {
              let boatTypeA = (boatTypes.find(t => t.id === a.boattype));
              let boatTypeAStr = boatTypeA === undefined ? "" : boatTypeA.name;
              let boatTypeB = boatTypes.find(t => t.id === b.boattype);
              let boatTypeBStr = boatTypeB === undefined ? "" : boatTypeB.name;
              return boatTypeAStr.localeCompare(boatTypeBStr);
            }
            else if(sortByStatus && sortByStatusAsc) {
              return a.status - b.status;
            }
            else if(sortByStatus && !sortByStatusAsc) {
              return b.status - a.status;
            }
            //Standard is sorting by name
            else {
              return a.name.localeCompare(b.name);
            }
          })
          .filter((boat) => boat.name.toLocaleLowerCase().includes(boatNameSearch.toLocaleLowerCase()))
          .filter((boat) => boatTypes.find(t => t.id === boat.boattype)?.name.toLocaleLowerCase().includes(boatTypeSearch.toLocaleLowerCase()))
          .filter((boat) => (boatStatusSearch === -1) ? true : (boat.status === boatStatusSearch))
          .map((x, i) => (
            <tr key={x.id}>
              <td>{x.name}</td>
              <td>{boatTypes.find(t => t.id === x.boattype)?.name}</td>
              <td><StatusIcon status={x.status}></StatusIcon></td>
              <td>
                <div className="d-flex">
                  <div
                    className="mx-2"
                    onClick={() => {
                      // TODO Check Problems with Status when editing ? 
                      setEditElement(x);
                      setValue('id', x.id);
                      setValue('name', x.name);
                      setValue('status', x.status);
                    }}
                  >
                    <FontAwesomeIcon
                      icon={faEdit}
                      className="text-secondary clickableIcon"
                      size="1x"
                    />
                  </div>
                  <div
                    className="mx-2"
                    onClick={() => {
                      setDeleteElement(boats.indexOf(x));
                    }}
                  >
                    <FontAwesomeIcon
                      icon={faTrashAlt}
                      className="text-danger clickableIcon"
                      size="1x"
                    />
                  </div>
                </div>
              </td>
            </tr>
          ))}
        </tbody>
      </Table>
      <div
        className="d-flex px-2 py-1 justify-content-end bg-white border-top"
        style={{ position: "sticky", right: "5px", bottom: "0", zIndex: 3 }}
      >
        <Button
          onClick={() => {
            setAdding(true);
            setEditElement({ name: "" } as Boat);
            setValue('id', undefined as any);
            setValue('name', "");
            setValue('status', 0);
          }}
          variant="secondary"
        >
          <FontAwesomeIcon icon={faPlus} className="text-white me-2" />
          {t("boatManager.AddBoat")}
        </Button>
      </div>


      <StaffModal
        header={isAdding ? t("boatManager.AddBoat") : t("boatManager.EditBoat")}
        show={!!editElement}
        successText={isAdding ? t("common.Add") : t("common.Edit")}
        loadingNext={loading}
        disableNext={!isValid}
        onHide={() => {
          setEditElement(undefined);
          setAdding(false);
        }}
        onSuccess={() => {
          setLoading(true);
          handleSubmit((data) => {
            if (isAdding) {
              addBoat(data).then(async () => {
                await getBoats();
                setEditElement(undefined);
                setAdding(false);
                setLoading(false);
              });
            } else {
              editBoat(data).then(async () => {
                await getBoats();
                setEditElement(undefined);
                setAdding(false);
                setLoading(false);
              });
            }
          })()
        }}
      >
        <Form>
          {!isAdding && <Controller
            name="id"
            control={control}
            defaultValue={editElement?.id}
            render={({ field }) => (
              <div className="mb-2">
                <Form.Label>{t("boatManager.Id")}</Form.Label>
                <Form.Control disabled type="text" {...field} />
              </div>
            )}
          />}
          <Controller
            name="name"
            control={control}
            defaultValue={editElement?.name}
            render={({ field }) => (
              <div className="mb-2">
                <Form.Label>{t("boatManager.BoatName")}</Form.Label>
                <Form.Control type="text" {...field} isInvalid={!!errors.name} />
                <Form.Control.Feedback type="invalid">
                  {errors.name?.message}
                </Form.Control.Feedback>
              </div>
            )}
            rules={{
              required: {
                value: true,
                message: t("common.messages.required", {
                  val: t("boatManager.BoatName"),
                }),
              },
            }}
          />
          <Controller
            name="boattype"
            control={control}
            defaultValue={editElement?.boattype}
            render={({ field }) => (
              <div className="mb-2">
                <Form.Label>{t("boatManager.BoatType")}</Form.Label>
                <Form.Select aria-label="Default select example" {...field} isInvalid={!!errors.boattype}>
                  <option></option>
                  {boatTypes.map((x) => (
                    <option key={x.id} value={x.id}>{x.name}</option>
                  ))}
                </Form.Select>
                <Form.Control.Feedback type="invalid">
                  {errors.boattype?.message}
                </Form.Control.Feedback>
              </div>
            )}
            rules={{
              required: {
                value: true,
                message: t("common.messages.required", {
                  val: t("boatManager.BoatType"),
                }),
              },
            }}
          />
          <Controller
            name="status"
            control={control}
            defaultValue={editElement?.status}
            render={({ field }) => (
              <div className="mb-2">
                <Form.Label>{t("boatManager.Status")}</Form.Label>
                <Form.Select aria-label="Default select example" {...field}>
                  <option value={0}>{t("boatManager.available")}</option>
                  <option value={1}>{t("boatManager.locked")}</option>
                </Form.Select>
              </div>
            )}
          />
        </Form>

      </StaffModal>


      <StaffModal
        header={t("boatManager.DeleteBoat")}
        hideColor="secondary"
        successText={t("common.Delete")}
        successColor="danger"
        show={deleteElement !== -1}
        loadingNext={loading}
        onHide={() => {
          setDeleteElement(-1);
        }}
        onSuccess={async () => {
          setLoading(true);
          await deleteBoat({ id: boats[deleteElement].id })
          await getBoats();
          setDeleteElement(-1);
          setLoading(false);
        }}
      >
        <span>
          {t("boatManager.messages.DeleteText", { val: boats[deleteElement]?.name })}
        </span>
      </StaffModal>
    </div>
  );
}

export default BoatManager;
